@@ -20,6 +20,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourceencoding.h"
 #include "gtksourceencoding-private.h"
 #include "gtksourceview-i18n.h"
@@ -52,7 +56,7 @@ G_DEFINE_BOXED_TYPE (GtkSourceEncoding, gtk_source_encoding,
  * Copyright (C) 2002 Red Hat, Inc.
  */
 
-typedef enum
+typedef enum _GtkSourceEncodingIndex
 {
 	GTK_SOURCE_ENCODING_ISO_8859_1,
 	GTK_SOURCE_ENCODING_ISO_8859_2,
@@ -653,8 +657,7 @@ gtk_source_encoding_get_default_candidates (void)
 			   encodings_str_translated,
 			   error->message);
 
-		g_error_free (error);
-		error = NULL;
+		g_clear_error (&error);
 
 		encodings_variant = g_variant_parse (G_VARIANT_TYPE_STRING_ARRAY,
 						     encodings_str,
@@ -665,11 +668,9 @@ gtk_source_encoding_get_default_candidates (void)
 		g_assert_no_error (error);
 	}
 
-	g_variant_ref_sink (encodings_variant);
-
 	encodings_strv = g_variant_get_strv (encodings_variant, NULL);
 	encodings_list = strv_to_list (encodings_strv);
-	g_free (encodings_strv);
+	g_free ((gpointer) encodings_strv);
 
 	/* Ensure that UTF-8 and CURRENT are present. */
 	encodings_list = g_slist_prepend (encodings_list,
