@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- * gtksourceengine.h - Abstract base class for highlighting engines
+ * gtksourceengine.h
  * This file is part of GtkSourceView
  *
  * Copyright (C) 2003 - Gustavo Giráldez
@@ -19,8 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GTK_SOURCE_ENGINE_H__
-#define __GTK_SOURCE_ENGINE_H__
+#ifndef GTK_SOURCE_ENGINE_H
+#define GTK_SOURCE_ENGINE_H
 
 #include <gtk/gtk.h>
 #include "gtksourcetypes.h"
@@ -56,10 +56,6 @@ struct _GtkSourceEngineInterface
 
 	void     (* set_style_scheme) (GtkSourceEngine      *engine,
 				       GtkSourceStyleScheme *scheme);
-
-	GtkTextTag *(* get_context_class_tag)
-				      (GtkSourceEngine      *engine,
-				       const gchar          *context_class);
 };
 
 G_GNUC_INTERNAL
@@ -89,11 +85,6 @@ G_GNUC_INTERNAL
 void        _gtk_source_engine_set_style_scheme	(GtkSourceEngine      *engine,
 						 GtkSourceStyleScheme *scheme);
 
-G_GNUC_INTERNAL
-GtkTextTag *_gtk_source_engine_get_context_class_tag
-						 (GtkSourceEngine     *engine,
-						  const gchar         *context_class);
-
 G_END_DECLS
 
-#endif /* __GTK_SOURCE_ENGINE_H__ */
+#endif /* GTK_SOURCE_ENGINE_H */
