@@ -19,8 +19,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourcestylescheme.h"
 #include "gtksourcestyleschememanager.h"
+#include "gtksourcestyle.h"
 #include "gtksourcestyle-private.h"
 #include "gtksourceview.h"
 #include "gtksourcelanguage-private.h"
@@ -37,10 +42,14 @@
  * #GtkSourceStyleScheme contains all the text styles to be used in
  * #GtkSourceView and #GtkSourceBuffer. For instance, it contains text styles
  * for syntax highlighting, it may contain foreground and background color for
- * non-highlighted text, color for the line numbers, etc.
+ * non-highlighted text, color for the line numbers, current line highlighting,
+ * bracket matching, etc.
  *
  * Style schemes are stored in XML files. The format of a scheme file is
  * documented in the [style scheme reference][style-reference].
+ *
+ * The two style schemes with IDs "classic" and "tango" follow more closely the
+ * GTK+ theme (for example for the background color).
  */
 
 #define STYLE_TEXT			"text"
@@ -655,52 +664,72 @@ get_cursors_css_style (GtkSourceStyleScheme *scheme,
 	GtkSourceStyle *secondary_style;
 	GdkRGBA primary_color = { 0 };
 	GdkRGBA secondary_color = { 0 };
-	gchar *primary_color_str;
+	gboolean primary_color_set;
+	gboolean secondary_color_set;
 	gchar *secondary_color_str;
-	gchar *css;
+	GString *css;
 
 	primary_style = gtk_source_style_scheme_get_style (scheme, STYLE_CURSOR);
 	secondary_style = gtk_source_style_scheme_get_style (scheme, STYLE_SECONDARY_CURSOR);
 
-	if (!get_color (primary_style, TRUE, &primary_color))
+	primary_color_set = get_color (primary_style, TRUE, &primary_color);
+	secondary_color_set = get_color (secondary_style, TRUE, &secondary_color);
+
+	if (!primary_color_set && !secondary_color_set)
 	{
 		return NULL;
 	}
 
-	if (!get_color (secondary_style, TRUE, &secondary_color))
+	css = g_string_new ("textview text {\n");
+
+	if (primary_color_set)
+	{
+		gchar *primary_color_str;
+
+		primary_color_str = gdk_rgba_to_string (&primary_color);
+		g_string_append_printf (css,
+					"\tcaret-color: %s;\n",
+					primary_color_str);
+		g_free (primary_color_str);
+	}
+
+	if (!secondary_color_set)
 	{
 		GtkStyleContext *context;
-		GdkRGBA *rgba;
+		GdkRGBA *background_color;
+
+		g_assert (primary_color_set);
 
 		context = gtk_widget_get_style_context (widget);
 
+		gtk_style_context_save (context);
+		gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+
 		gtk_style_context_get (context,
-				       GTK_STATE_FLAG_NORMAL,
-				       "background-color", &rgba,
+				       gtk_style_context_get_state (context),
+				       "background-color", &background_color,
 				       NULL);
 
-		/* shade the secondary cursor */
-		secondary_color.red = rgba->red * 0.5;
-		secondary_color.green = rgba->green * 0.5;
-		secondary_color.blue = rgba->blue * 0.5;
+		gtk_style_context_restore (context);
 
-		gdk_rgba_free (rgba);
+		/* Blend primary cursor color with background color. */
+		secondary_color.red = (primary_color.red + background_color->red) * 0.5;
+		secondary_color.green = (primary_color.green + background_color->green) * 0.5;
+		secondary_color.blue = (primary_color.blue + background_color->blue) * 0.5;
+		secondary_color.alpha = (primary_color.alpha + background_color->alpha) * 0.5;
+
+		gdk_rgba_free (background_color);
 	}
 
-	primary_color_str = gdk_rgba_to_string (&primary_color);
 	secondary_color_str = gdk_rgba_to_string (&secondary_color);
-
-	css = g_strdup_printf ("GtkSourceView {\n"
-			       "\t-GtkWidget-cursor-color: %s;\n"
-			       "\t-GtkWidget-secondary-cursor-color: %s;\n"
-			       "}\n",
-			       primary_color_str,
-			       secondary_color_str);
-
-	g_free (primary_color_str);
+	g_string_append_printf (css,
+				"\t-gtk-secondary-caret-color: %s;\n",
+				secondary_color_str);
 	g_free (secondary_color_str);
 
-	return css;
+	g_string_append_printf (css, "}\n");
+
+	return g_string_free (css, FALSE);
 }
 
 /* The CssProvider for the cursors depends only on @scheme, but it needs a
@@ -733,10 +762,8 @@ get_css_provider_cursors (GtkSourceStyleScheme *scheme,
 	if (error != NULL)
 	{
 		g_warning ("Error when loading CSS for cursors: %s", error->message);
-		g_error_free (error);
-
-		g_object_unref (provider);
-		return NULL;
+		g_clear_error (&error);
+		g_clear_object (&provider);
 	}
 
 	return provider;
@@ -745,23 +772,22 @@ get_css_provider_cursors (GtkSourceStyleScheme *scheme,
 /**
  * _gtk_source_style_scheme_apply:
  * @scheme:: a #GtkSourceStyleScheme.
- * @widget: a #GtkWidget to apply styles to.
+ * @view: a #GtkSourceView to apply styles to.
  *
- * Sets text colors from @scheme in the @widget.
+ * Sets style colors from @scheme to the @view.
  *
  * Since: 2.0
  */
 void
 _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
-				GtkWidget            *widget)
+				GtkSourceView        *view)
 {
 	GtkStyleContext *context;
 
 	g_return_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme));
-	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (GTK_SOURCE_IS_VIEW (view));
 
-	/* we need to translate some of the style scheme properties in a CSS override */
-	context = gtk_widget_get_style_context (GTK_WIDGET (widget));
+	context = gtk_widget_get_style_context (GTK_WIDGET (view));
 	gtk_style_context_add_provider (context,
 	                                GTK_STYLE_PROVIDER (scheme->priv->css_provider),
 	                                GTK_SOURCE_STYLE_PROVIDER_PRIORITY);
@@ -776,7 +802,8 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 	 */
 	if (scheme->priv->css_provider_cursors == NULL)
 	{
-		scheme->priv->css_provider_cursors = get_css_provider_cursors (scheme, widget);
+		scheme->priv->css_provider_cursors = get_css_provider_cursors (scheme,
+									       GTK_WIDGET (view));
 	}
 
 	if (scheme->priv->css_provider_cursors != NULL)
@@ -793,23 +820,23 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 
 /**
  * _gtk_source_style_scheme_unapply:
- * @scheme: (allow-none): a #GtkSourceStyleScheme or %NULL.
- * @widget: a #GtkWidget to unapply styles to.
+ * @scheme: (nullable): a #GtkSourceStyleScheme or %NULL.
+ * @view: a #GtkSourceView to unapply styles to.
  *
- * Removes the style from @scheme in the @widget.
+ * Removes the styles from @scheme in the @view.
  *
  * Since: 3.0
  */
 void
 _gtk_source_style_scheme_unapply (GtkSourceStyleScheme *scheme,
-                                  GtkWidget            *widget)
+				  GtkSourceView        *view)
 {
 	GtkStyleContext *context;
 
 	g_return_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme));
-	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (GTK_SOURCE_IS_VIEW (view));
 
-	context = gtk_widget_get_style_context (GTK_WIDGET (widget));
+	context = gtk_widget_get_style_context (GTK_WIDGET (view));
 	gtk_style_context_remove_provider (context,
 	                                   GTK_STYLE_PROVIDER (scheme->priv->css_provider));
 
@@ -894,29 +921,39 @@ generate_css_style (GtkSourceStyleScheme *scheme)
 	final_style = g_string_new ("");
 
 	style = gtk_source_style_scheme_get_style (scheme, STYLE_TEXT);
-	append_css_style (final_style, style, ".view");
+	append_css_style (final_style, style, "textview text");
 
 	style = gtk_source_style_scheme_get_style (scheme, STYLE_SELECTED);
-	append_css_style (final_style, style, ".view:selected:focused");
+	append_css_style (final_style, style, "textview:focus text selection");
 
 	style2 = gtk_source_style_scheme_get_style (scheme, STYLE_SELECTED_UNFOCUSED);
 	append_css_style (final_style,
 			  style2 != NULL ? style2 : style,
-			  ".view:selected");
+			  "textview text selection");
 
 	/* For now we use "line numbers" colors for all the gutters */
 	style = gtk_source_style_scheme_get_style (scheme, STYLE_LINE_NUMBERS);
 	if (style != NULL)
 	{
-		append_css_style (final_style, style, ".top");
-		append_css_style (final_style, style, ".right");
-		append_css_style (final_style, style, ".bottom");
-		append_css_style (final_style, style, ".left");
+		append_css_style (final_style, style, "textview border");
+
+		/* Needed for GtkSourceGutter. In the ::draw callback,
+		 * gtk_style_context_add_class() is called to add e.g. the
+		 * "left" class. Because as of GTK+ 3.20 we cannot do the same
+		 * to add the "border" subnode.
+		 */
+		append_css_style (final_style, style, "textview .left");
+		append_css_style (final_style, style, "textview .right");
+		append_css_style (final_style, style, "textview .top");
+		append_css_style (final_style, style, "textview .bottom");
 
 		/* For the corners if the top or bottom gutter is also
 		 * displayed.
+		 * FIXME: this shouldn't be necessary, GTK+ should apply the
+		 * border style to the corners too, see:
+		 * https://bugzilla.gnome.org/show_bug.cgi?id=764239
 		 */
-		append_css_style (final_style, style, "GtkSourceView");
+		append_css_style (final_style, style, "textview");
 	}
 
 	style = gtk_source_style_scheme_get_style (scheme, STYLE_CURRENT_LINE_NUMBER);
@@ -929,13 +966,15 @@ generate_css_style (GtkSourceStyleScheme *scheme)
 	{
 		GError *error = NULL;
 
-		if (!gtk_css_provider_load_from_data (scheme->priv->css_provider,
-						      final_style->str,
-		                                      final_style->len,
-						      &error))
+		gtk_css_provider_load_from_data (scheme->priv->css_provider,
+						 final_style->str,
+						 final_style->len,
+						 &error);
+
+		if (error != NULL)
 		{
 			g_warning ("%s", error->message);
-			g_error_free (error);
+			g_clear_error (&error);
 		}
 	}
 
@@ -1002,16 +1041,6 @@ parse_style (GtkSourceStyleScheme *scheme,
 	tmp = xmlGetProp (node, BAD_CAST "use-style");
 	if (tmp != NULL)
 	{
-		if (use_style != NULL)
-		{
-			g_set_error (error, ERROR_QUARK, 0,
-				     "in style '%s': duplicated use-style attribute",
-				     style_name);
-			g_free (style_name);
-			g_object_unref (use_style);
-			return FALSE;
-		}
-
 		use_style = gtk_source_style_scheme_get_style (scheme, (char*) tmp);
 
 		if (use_style == NULL)
@@ -1326,7 +1355,7 @@ _gtk_source_style_scheme_new_from_file (const gchar *filename)
 		g_warning ("could not load style scheme file '%s': %s",
 			   filename_utf8, error->message);
 		g_free (filename_utf8);
-		g_error_free (error);
+		g_clear_error (&error);
 		return NULL;
 	}
 
@@ -1364,9 +1393,8 @@ _gtk_source_style_scheme_new_from_file (const gchar *filename)
 		g_warning ("could not load style scheme file '%s': %s",
 			   filename_utf8, error->message);
 		g_free (filename_utf8);
-		g_error_free (error);
-		g_object_unref (scheme);
-		scheme = NULL;
+		g_clear_error (&error);
+		g_clear_object (&scheme);
 	}
 	else
 	{
